<?php

namespace Drupal\access_policy;

use Drupal\access_policy\Entity\AccessPolicyInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Access rule plugin manager.
 */
class AccessRulePluginManager extends DefaultPluginManager {

  /**
   * AccessRulePluginManager constructor.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/AccessRule', $namespaces, $module_handler, 'Drupal\access_policy\Plugin\AccessRule\AccessRulePluginInterface', 'Drupal\access_policy\Annotation\AccessRule');

    $this->alterInfo('access_rule');
    $this->setCacheBackend($cache_backend, 'access_rule');
  }

  /**
   * Get the applicable access rule plugins.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $policy
   *   The current access policy.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   *
   * @return array
   *   Array of access rule plugins.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getApplicableAccessRulePlugins(AccessPolicyInterface $policy, EntityInterface $entity) {
    $applicable_plugins = [];
    $rules = $policy->getAccessRules();
    foreach ($rules as $id => $plugin_config) {
      $plugin = $this->createInstance($plugin_config['plugin_id']);
      $plugin_settings = $policy->getAccessRule($id);
      $settings = $plugin_settings['settings'] ?? [];
      $plugin->setAccessPolicy($policy);
      $plugin->setConfiguration($settings);

      if ($plugin->isApplicable($entity)) {
        $applicable_plugins[$id] = $plugin;
      }
    }

    return $applicable_plugins;
  }

}
