<?php

namespace Drupal\Tests\access_policy\Functional;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;

/**
 * Base class for access policy functional tests.
 *
 * This is primarily added to address an issue with drupalLogin.
 */
class AccessPolicyTestBase extends BrowserTestBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The access policy entity storage.
   *
   * @var \Drupal\access_policy\ContentAccessPolicyManager
   */
  protected $contentAccessPolicyManager;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setup();

    $this->entityTypeManager = \Drupal::service('entity_type.manager');
    $this->contentAccessPolicyManager = \Drupal::service('access_policy.content_policy_manager');
  }

  /**
   * Helper function for assigning an access policy to a node.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $access_policy
   *   The access policy.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function assignAccessPolicy(EntityInterface $entity, string $access_policy) {
    $access_policy = $this->entityTypeManager->getStorage('access_policy')->load($access_policy);
    $this->contentAccessPolicyManager->assign($entity, $access_policy);
  }

  /**
   * {@inheritdoc}
   */
  protected function drupalLogin(AccountInterface $account) {
    if ($this->loggedInUser) {
      $this->drupalLogout();
    }

    $this->drupalGet(Url::fromRoute('user.login'));
    $this->submitForm([
      'name' => $account->getAccountName(),
      'pass' => $account->passRaw,
    ], 'Log in');

    // @see ::drupalUserIsLoggedIn()
    $account->sessionId = $this->getSession()->getCookie(\Drupal::service('session_configuration')->getOptions(\Drupal::request())['name']);
    // @todo this is failing for some reason.
    //$this->assertTrue($this->drupalUserIsLoggedIn($account), new FormattableMarkup('User %name successfully logged in.', ['%name' => $account->getAccountName()]));

    $this->loggedInUser = $account;
    $this->container->get('current_user')->setAccount($account);
  }

}
